/*
 ** Copyright(C) 2004,2005 INL
 ** Written by Eric Leblond <regit@inl.fr>
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
 ** the Free Software Foundation, version 2 of the License.
 **
 ** This program is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
 ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ** GNU General Public License for more details.
 **
 ** You should have received a copy of the GNU General Public License
 ** along with this program; if not, write to the Free Software
 ** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>

#include <gnome.h>

#include "interface.h"
#include "nuapplet.h"

GtkWidget*
create_settings (void)
{
  GtkWidget *settings;
  GtkWidget *notebook1;
  GtkWidget *table1;
  GtkWidget *user_label;
  GtkWidget *host_label;
  GtkWidget *hbox1;
  GtkWidget *port_label;
  GtkObject *port_entry_adj;
  GtkWidget *port_entry;
  GtkWidget *user_entry;
  GtkWidget *host_entry;
  GtkWidget *user_settings;
  GtkWidget *table2;
  GtkWidget *interval_label;
  GtkObject *interval_entry_adj;
  GtkWidget *interval_entry;
  GtkWidget *log_at_start;
  GtkWidget *advanced_settings;

  settings = gnome_property_box_new ();
  gtk_object_set_data (GTK_OBJECT (settings), "settings", settings);
  gtk_window_set_policy (GTK_WINDOW (settings), FALSE, FALSE, FALSE);

  notebook1 = GNOME_PROPERTY_BOX (settings)->notebook;
  gtk_object_set_data (GTK_OBJECT (settings), "notebook1", notebook1);
  gtk_widget_show (notebook1);

  table1 = gtk_table_new (2, 3, FALSE);
  gtk_widget_ref (table1);
  gtk_object_set_data_full (GTK_OBJECT (settings), "table1", table1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (table1);
  gtk_container_add (GTK_CONTAINER (notebook1), table1);

  user_label = gtk_label_new (_("User: "));
  gtk_widget_ref (user_label);
  gtk_object_set_data_full (GTK_OBJECT (settings), "user_label", user_label,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (user_label);
  gtk_table_attach (GTK_TABLE (table1), user_label, 0, 1, 0, 1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (user_label), 0, 0.5);

  host_label = gtk_label_new (_("NuGateway: "));
  gtk_widget_ref (host_label);
  gtk_object_set_data_full (GTK_OBJECT (settings), "host_label", host_label,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (host_label);
  gtk_table_attach (GTK_TABLE (table1), host_label, 0, 1, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (host_label), 0, 0.5);

  hbox1 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox1);
  gtk_object_set_data_full (GTK_OBJECT (settings), "hbox1", hbox1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox1);
  gtk_table_attach (GTK_TABLE (table1), hbox1, 2, 3, 1, 2,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 0, 0);

  port_label = gtk_label_new (_("Port: "));
  gtk_widget_ref (port_label);
  gtk_object_set_data_full (GTK_OBJECT (settings), "port_label", port_label,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (port_label);
  gtk_box_pack_start (GTK_BOX (hbox1), port_label, FALSE, FALSE, 0);

  port_entry_adj = gtk_adjustment_new (nuapplet->port, 0, 65536, 1, 10, 10);
  port_entry = gtk_spin_button_new (GTK_ADJUSTMENT (port_entry_adj), 1, 0);
  gtk_widget_ref (port_entry);
  gtk_object_set_data_full (GTK_OBJECT (settings), "port_entry", port_entry,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_signal_connect (GTK_OBJECT (port_entry), "value-changed",
                      GTK_SIGNAL_FUNC (on_modif_do),
                      NULL);
  gtk_widget_show (port_entry);
  gtk_box_pack_start (GTK_BOX (hbox1), port_entry, TRUE, TRUE, 0);
  gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (port_entry), TRUE);
  gtk_spin_button_set_update_policy (GTK_SPIN_BUTTON (port_entry), GTK_UPDATE_IF_VALID);

  user_entry = gtk_entry_new ();
  gtk_widget_ref (user_entry);
  gtk_object_set_data_full (GTK_OBJECT (settings), "user_entry", user_entry,
                            (GtkDestroyNotify) gtk_widget_unref);
  if (nuapplet->username)
	gtk_entry_set_text(GTK_ENTRY(user_entry),nuapplet->username);

  gtk_signal_connect (GTK_OBJECT (user_entry), "changed",
                      GTK_SIGNAL_FUNC (on_modif_do),
                      NULL);
  gtk_widget_show (user_entry);
  gtk_table_attach (GTK_TABLE (table1), user_entry, 1, 2, 0, 1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  host_entry = gtk_entry_new ();
  gtk_widget_ref (host_entry);
  gtk_object_set_data_full (GTK_OBJECT (settings), "host_entry", host_entry,
                            (GtkDestroyNotify) gtk_widget_unref);
  if (nuapplet->host){
  gtk_entry_set_text(GTK_ENTRY(host_entry),nuapplet->host);
		  }
gtk_entry_set_activates_default (GTK_ENTRY(host_entry),TRUE);
  gtk_signal_connect (GTK_OBJECT (host_entry), "changed",
                      GTK_SIGNAL_FUNC (on_modif_do),
                      NULL);
  gtk_widget_show (host_entry);
  gtk_table_attach (GTK_TABLE (table1), host_entry, 1, 2, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  user_settings = gtk_label_new (_("Global Settings"));
  gtk_widget_ref (user_settings);
  gtk_object_set_data_full (GTK_OBJECT (settings), "user_settings", user_settings,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (user_settings);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook1), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook1), 0), user_settings);

/* interface settings */ 

  table2 = gtk_table_new (2, 4, FALSE);
  gtk_widget_ref (table2);
  gtk_object_set_data_full (GTK_OBJECT (settings), "table2", table2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (table2);
  gtk_container_add (GTK_CONTAINER (notebook1), table2);

  interval_label = gtk_label_new (_("Interval :"));
  gtk_widget_ref (interval_label);
  gtk_object_set_data_full (GTK_OBJECT (settings), "interval_label", interval_label,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (interval_label);
  gtk_table_attach (GTK_TABLE (table2), interval_label, 0, 1, 0, 1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_label_set_justify (GTK_LABEL (interval_label), GTK_JUSTIFY_RIGHT);
  gtk_misc_set_alignment (GTK_MISC (interval_label), 0, 0.5);

 interval_entry_adj = gtk_adjustment_new (nuapplet->interval, 100, 1000, 10, 10, 10);
  interval_entry = gtk_spin_button_new (GTK_ADJUSTMENT (interval_entry_adj), 1, 0);
  gtk_widget_ref (interval_entry);
  gtk_object_set_data_full (GTK_OBJECT (settings), "interval_entry", interval_entry,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_signal_connect (GTK_OBJECT (interval_entry), "changed",
                      GTK_SIGNAL_FUNC (on_modif_do),
                      NULL);
  gtk_widget_show (interval_entry);
  gtk_table_attach (GTK_TABLE (table2), interval_entry, 1, 2, 0, 1,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (interval_entry), TRUE);

log_at_start = gtk_check_button_new_with_label (_("Log at start"));
  gtk_widget_ref (log_at_start);
  gtk_object_set_data_full (GTK_OBJECT (settings), "log_at_start", log_at_start,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(log_at_start),nuapplet->log_at_start);
  gtk_widget_show (log_at_start);
  gtk_table_attach (GTK_TABLE (table2), log_at_start, 0, 1, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  advanced_settings = gtk_label_new (_("Interface Settings"));
  gtk_widget_ref (advanced_settings);
  gtk_object_set_data_full (GTK_OBJECT (settings), "interface_settings", advanced_settings,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (advanced_settings);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook1), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook1), 1), advanced_settings);


  gtk_widget_grab_focus (user_entry);
  gtk_signal_connect (GTK_OBJECT (settings), "apply",
                      GTK_SIGNAL_FUNC (on_settings_apply),
                      NULL);

  return settings;
}

GtkWidget*
create_password_dialog (void)
{
  GtkWidget *password_dialog;
  GtkWidget *dialog_vbox1;
  GtkWidget *table3;
  GtkWidget *user_entry;
  GtkWidget *user_dialog_label;
  GtkWidget *label_dialog_password;
  GtkWidget *password_entry;
  GtkWidget *dialog_action_area1;

  password_dialog = gnome_dialog_new (_("Enter NuFW password"), GNOME_STOCK_BUTTON_OK, GNOME_STOCK_BUTTON_CANCEL, NULL);
  gtk_object_set_data (GTK_OBJECT (password_dialog), "password_dialog", password_dialog);
  gtk_window_set_policy (GTK_WINDOW (password_dialog), FALSE, FALSE, FALSE);

  dialog_vbox1 = GNOME_DIALOG (password_dialog)->vbox;
  gtk_object_set_data (GTK_OBJECT (password_dialog), "dialog_vbox1", dialog_vbox1);
  gtk_widget_show (dialog_vbox1);

  table3 = gtk_table_new (2, 2, FALSE);
  gtk_widget_ref (table3);
  gtk_object_set_data_full (GTK_OBJECT (password_dialog), "table3", table3,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (table3);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), table3, TRUE, TRUE, 0);

  user_entry = gtk_entry_new ();
  gtk_widget_ref (user_entry);
  gtk_object_set_data_full (GTK_OBJECT (password_dialog), "user_entry", user_entry,
		  (GtkDestroyNotify) gtk_widget_unref);
  if (nuapplet->username)
	  gtk_entry_set_text(GTK_ENTRY(user_entry),nuapplet->username);

  gtk_widget_show (user_entry);
  gtk_table_attach (GTK_TABLE (table3), user_entry, 1, 2, 0, 1,
                    (GtkAttachOptions) (GTK_EXPAND),
                    (GtkAttachOptions) (0), 0, 0);

  user_dialog_label = gtk_label_new (_("User: "));
  gtk_widget_ref (user_dialog_label);
  gtk_object_set_data_full (GTK_OBJECT (password_dialog), "user_dialog_label", user_dialog_label,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (user_dialog_label);
  gtk_table_attach (GTK_TABLE (table3), user_dialog_label, 0, 1, 0, 1,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_label_set_justify (GTK_LABEL (user_dialog_label), GTK_JUSTIFY_RIGHT);
  gtk_misc_set_alignment (GTK_MISC (user_dialog_label), 0, 0.5);

  label_dialog_password = gtk_label_new (_("Password: "));
  gtk_widget_ref (label_dialog_password);
  gtk_object_set_data_full (GTK_OBJECT (password_dialog), "label_dialog_password", label_dialog_password,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label_dialog_password);
  gtk_table_attach (GTK_TABLE (table3), label_dialog_password, 0, 1, 1, 2,
                    (GtkAttachOptions) (GTK_EXPAND),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_label_set_justify (GTK_LABEL (label_dialog_password), GTK_JUSTIFY_RIGHT);
  gtk_misc_set_alignment (GTK_MISC (label_dialog_password), 0, 0.5);

  password_entry = gtk_entry_new ();
  gtk_widget_ref (password_entry);
  gtk_object_set_data_full (GTK_OBJECT (password_dialog), "password_entry", password_entry,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (password_entry);
  gtk_table_attach (GTK_TABLE (table3), password_entry, 1, 2, 1, 2,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_entry_set_visibility (GTK_ENTRY (password_entry), FALSE);
  if (nuapplet->password){
 	 gtk_entry_set_text(GTK_ENTRY (password_entry) , nuapplet->password);
  }

  gtk_entry_set_activates_default (GTK_ENTRY(password_entry),TRUE);
  dialog_action_area1 = GNOME_DIALOG (password_dialog)->action_area;
  gtk_object_set_data (GTK_OBJECT (password_dialog), "dialog_action_area1", dialog_action_area1);
  gtk_widget_show (dialog_action_area1);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area1), GTK_BUTTONBOX_END);
  gtk_button_box_set_spacing (GTK_BUTTON_BOX (dialog_action_area1), 8);
  
  gtk_widget_grab_focus (password_entry);
  
gnome_dialog_button_connect(GNOME_DIALOG(password_dialog),0,
                      GTK_SIGNAL_FUNC (on_password_start),
                      NULL);
gnome_dialog_button_connect(GNOME_DIALOG(password_dialog),1,
                      GTK_SIGNAL_FUNC (on_password_cancel),
                      NULL);

  gnome_dialog_set_default (GNOME_DIALOG(password_dialog),0);

  return password_dialog;
}

