/*
 ** Copyright(C) 2004,2005 INL
 ** Written by Eric Leblond <regit@inl.fr>
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
 ** the Free Software Foundation, version 2 of the License.
 **
 ** This program is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
 ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ** GNU General Public License for more details.
 **
 ** You should have received a copy of the GNU General Public License
 ** along with this program; if not, write to the Free Software
 ** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include <string.h>

#include <gnome.h>
#include <gtk/gtklabel.h>
#include <gtk/gtkbox.h>
#include <gtk/gtkimage.h>
#include <gtk/gtk.h>

#include "nuapplet.h"
#include "interface.h"

gboolean stop_auth(NuApplet *nuapplet);
gboolean start_auth(NuApplet *nuapplet);

char * get_username()
{
	return g_strdup(nuapplet->username);
}


char* get_password()
{
	return g_strdup(nuapplet->password);
}



/* Here some rearangement of the icons and the labels occurs
 * according to the panelsize and wether we show in and out
 * or just the sum
 */
	static void
applet_change_size(PanelApplet *applet_widget, int size, gpointer data)
{
	gchar sizename[30];
	int panel_size = panel_applet_get_size(nuapplet->applet);
	int iconsize;

	snprintf(sizename,29,"panel size %d",panel_size);
	iconsize = gtk_icon_size_from_name (sizename);
	if (iconsize <= 0) {
		nuapplet->panel_size = gtk_icon_size_register     ( sizename    ,
				panel_size,panel_size);
	} else {
		nuapplet->panel_size = iconsize;
	}
	if (nuapplet->auth_on){
		gtk_image_set_from_stock (
				GTK_IMAGE(nuapplet->image),
				"nufw-active",
				nuapplet->panel_size	
				);
	} else {
		gtk_image_set_from_stock (
				GTK_IMAGE(nuapplet->image),
				"nufw-inactive",
				nuapplet->panel_size	
				);

	}

}

gboolean on_modif_do (GtkWidget      *event_box, 
		GdkEventButton *event,
		gpointer        data)
{
	gnome_property_box_changed(GNOME_PROPERTY_BOX(nuapplet->settings));
	return TRUE;
}

gboolean on_settings_apply (GtkWidget      *event_box, 
		GdkEventButton *event,
		gpointer        data) 
{
	/* get all values 
	 * get advanced options 
	* get interval */
	nuapplet->interval = gtk_spin_button_get_value_as_int (
			gtk_object_get_data(GTK_OBJECT(event_box),"interval_entry")
			);

	nuapplet->log_at_start = gtk_toggle_button_get_active (
			gtk_object_get_data(GTK_OBJECT(event_box),"log_at_start")
			);
	/* get standard options */
	/* user_entry */
        if (nuapplet->username){
                g_free(nuapplet->username);
        }
        nuapplet->username= g_strdup (gtk_entry_get_text(
				gtk_object_get_data(GTK_OBJECT(event_box),"user_entry")
				));
	/* host_entry */
	if (nuapplet->host)
		g_free(nuapplet->host);
	nuapplet->host = g_strdup (gtk_entry_get_text(
				gtk_object_get_data(GTK_OBJECT(event_box),"host_entry")
				));
	/* get port */
	nuapplet->port = gtk_spin_button_get_value_as_int (
			gtk_object_get_data(GTK_OBJECT(event_box),"port_entry")
			);



	/* store preferences */
	panel_applet_gconf_set_string(PANEL_APPLET(nuapplet->applet), "username", nuapplet->username, NULL);
	panel_applet_gconf_set_string(PANEL_APPLET(nuapplet->applet), "hostname", nuapplet->host, NULL);
	panel_applet_gconf_set_int(PANEL_APPLET(nuapplet->applet), "port", nuapplet->port, NULL);
	panel_applet_gconf_set_int(PANEL_APPLET(nuapplet->applet), "interval", nuapplet->interval, NULL);
	panel_applet_gconf_set_bool(PANEL_APPLET(nuapplet->applet), "log_at_start", nuapplet->log_at_start, NULL);

	panel_applet_gconf_set_bool(PANEL_APPLET(nuapplet->applet), "have_settings", TRUE, NULL);
	if (nuapplet->starting){
		nuapplet->starting = FALSE;
		nuapplet->password_dialog =create_password_dialog();
		gtk_widget_show( nuapplet->password_dialog );
	}
	if (nuapplet->auth_on == TRUE ){
		/* we do a restart if pref have changed */
		stop_auth(nuapplet);
		start_auth(nuapplet);
	}
	return TRUE;

}

static void 
display_properties_dialog (BonoboUIComponent *uic, PanelApplet *applet,
		const gchar *verbname) 
{
	nuapplet->settings = create_settings();
	gtk_widget_show( nuapplet->settings);
}

static void 
display_about_dialog (BonoboUIComponent *uic, PanelApplet *applet,
		const gchar *verbname) 
{
	static GtkWidget *about = NULL;
	GdkPixbuf *pixbuf;
	static const gchar *authors[] = {
		"Eric Leblond <regit@inl.fr>",
		NULL
	};

	const char *documenters [] = {
		NULL
	};

	const char *translator_credits = NULL;

	if (about)
	{
		gtk_window_present (GTK_WINDOW (about));
		return;
	}

	pixbuf = gdk_pixbuf_new_from_file_at_size (GNOMEICONDIR "/nufw_active.svg",78,78, NULL);
	about = gnome_about_new ("NuApplet", VERSION,
			"(C) 2004 INL",
			_("NuFW Client Applet for GNOME"),
			authors,
			documenters,
			translator_credits,
			pixbuf
			);

	g_signal_connect (G_OBJECT(about), "destroy",
			(GCallback)gtk_widget_destroyed, &about);

	gtk_widget_show (about);
}

static gboolean call_nu_client(NuApplet * nusession)
{
	if (nuapplet->auth_on == TRUE){
		if (nuapplet->session){
			if (nu_client_check(nuapplet->session
			#ifdef API_V2
			,NULL
			#endif
			)< 0){
				nuapplet->session = NULL;
				nuapplet->timeout=1000;
				nuapplet->timevalue=0;
			}
		} else {
			nuapplet->timevalue+=nuapplet->interval;
			if (nuapplet->timevalue >= nuapplet->timeout){
				nuapplet->session = nu_client_init2(
						nuapplet->host,
						nuapplet->port,
						NULL,
						NULL,
						&get_username,
						&get_password,
						NULL
#ifdef API_V2
						,NULL
#endif
						);
				if (nuapplet->session == NULL){
					if (nuapplet->timeout < RETRY_TIME_MAX)
						nuapplet->timeout=2*nuapplet->timeout;
					nuapplet->timevalue=0;
				}
			}
		}
	} else {
		if ( nuapplet->session ){
			nu_client_free(nuapplet->session
#ifdef API_V2
						,NULL
#endif

			);
			nuapplet->session = NULL;
		}
	}
	return TRUE;
}

gboolean stop_auth(NuApplet *nuapplet){
	if(nuapplet->auth_on == TRUE){
		nuapplet->auth_on=FALSE;
		g_source_remove(nuapplet->timer);
		gtk_image_set_from_stock (
				GTK_IMAGE(nuapplet->image),
				"nufw-inactive",
				nuapplet->panel_size	
				);
		gtk_tooltips_set_tip (nuapplet->tooltip,
				nuapplet->event_box,
				_("NuFW authentication inactive"),
				_("Click right to activate"));
	}
	return TRUE;
}

gboolean start_auth(NuApplet *nuapplet){
	if (nuapplet->auth_on == FALSE ){
#if 0
		if ( nuapplet->session ){
			nu_client_free(nuapplet->session);
			nuapplet->session = NULL;
		}
#endif
	nuapplet->session = nu_client_init2(
						nuapplet->host,
						nuapplet->port,
						NULL,
						NULL,
						&get_username,
						&get_password,
						NULL
#ifdef API_V2
						,NULL
#endif
					);

		if (nuapplet->session == NULL) {
			int nerror=errno;
			GtkWidget *info_dialog;
			char *string_error;
			if (nerror == EACCES){
				string_error=_("Invalid Login/Password");
			} else 
				string_error = strerror(nerror);
			info_dialog = gnome_message_box_new (
					g_strconcat (
						_("Connection failed :"),
						"\n",
						string_error,
						NULL),
					GNOME_MESSAGE_BOX_ERROR,
					"OK",
					NULL);
			gnome_dialog_run (GNOME_DIALOG (info_dialog));
		} else {
			nuapplet->auth_on= TRUE;
			nuapplet->timer = g_timeout_add(nuapplet->interval,call_nu_client,nuapplet);
			gtk_image_set_from_stock (
					GTK_IMAGE(nuapplet->image),
					"nufw-active",
					nuapplet->panel_size	
					);
			gtk_tooltips_set_tip (nuapplet->tooltip,
					nuapplet->event_box,
					_("NuFW authentication active"),
					_("Click right to inactivate"));
		}
	}
	return TRUE;
}

gboolean on_password_cancel (GtkWidget      *event_box, 
		GdkEventButton *event,
		gpointer        data) {
	gtk_widget_destroy(nuapplet->password_dialog);
	return TRUE;
}

gboolean on_password_start (GtkWidget      *event_box, 
		GdkEventButton *event,
		gpointer        data) 
{
	gchar * tmpchar;	
	if (nuapplet->password)
		g_free(nuapplet->password);

	/* user_entry */
	tmpchar = g_strdup (gtk_entry_get_text(
				GTK_ENTRY(
					gtk_object_get_data(GTK_OBJECT(nuapplet->password_dialog),"user_entry")
					)
				)
			);
	nuapplet->username=tmpchar;

	if (panel_applet_gconf_get_bool(PANEL_APPLET(nuapplet->applet), "have_settings", NULL))
	{
		panel_applet_gconf_set_string(PANEL_APPLET(nuapplet->applet), "username", nuapplet->username, NULL);
	}

	nuapplet->password = g_strdup (gtk_entry_get_text(
				gtk_object_get_data(GTK_OBJECT(nuapplet->password_dialog),"password_entry")
				));

	start_auth(nuapplet);

	gtk_widget_destroy(nuapplet->password_dialog);
	return TRUE;
}

static void 
launch_nu_client (BonoboUIComponent *uic, PanelApplet *applet,
		const gchar *verbname)
{

	if (! nuapplet->auth_on){
		if (nuapplet->host){
			nuapplet->password_dialog = create_password_dialog();
			gtk_widget_show(nuapplet->password_dialog);
		} else {
			nuapplet->starting = TRUE;
			nuapplet->settings = create_settings();
			gtk_widget_show(nuapplet->settings);
		}
	}
}

static void 
stop_nu_client (BonoboUIComponent *uic, PanelApplet *applet,
		const gchar *verbname)
{
	stop_auth(nuapplet);
}


static const BonoboUIVerb nu_menu_verbs [] = {
	BONOBO_UI_VERB ("Properties", display_properties_dialog),
	BONOBO_UI_VERB ("About", display_about_dialog),
	BONOBO_UI_VERB ("NuClientEnable", launch_nu_client),
	BONOBO_UI_VERB ("NuClientDisable", stop_nu_client),
	BONOBO_UI_VERB_END
};

	static gboolean
nu_applet_fill (PanelApplet *applet,
		const gchar *iid,
		gpointer data)
{
	GtkWidget *image, *event_box;
	GtkIconFactory* nufw_icons;
	GtkIconSet* nufw_icon = gtk_icon_set_new();
	GtkIconSet* nufw_icon_active = gtk_icon_set_new();
	GtkIconSource* src_nufw_icon = gtk_icon_source_new ();
	GtkIconSource* src_nufw_icon_active = gtk_icon_source_new ();
	if (strcmp (iid, "OAFIID:NuFWClientApplet"))
		return FALSE;

	nu_client_global_init(
#ifdef API_V2
			NULL
#endif
	);

	nuapplet = g_new0(NuApplet,1);
	nuapplet->auth_on= FALSE;
	nuapplet->starting = FALSE;
	nuapplet->applet=applet;
	nuapplet->port=DEFAULT_PORT;
	nuapplet->interval=DEFAULT_INTERVAL;
	nuapplet->log_at_start=FALSE;
	nuapplet->session = NULL;
	nuapplet->password= NULL;

	nufw_icons = gtk_icon_factory_new();

	gtk_icon_source_set_filename(src_nufw_icon, GNOMEICONDIR "/nufw.svg");

	gtk_icon_source_set_size_wildcarded (src_nufw_icon,
			TRUE);
	gtk_icon_source_set_filename(src_nufw_icon_active, GNOMEICONDIR "/nufw_active.svg");
	gtk_icon_source_set_size_wildcarded (src_nufw_icon_active,
			TRUE);

	gtk_icon_set_add_source(nufw_icon,src_nufw_icon);
	gtk_icon_set_ref(nufw_icon);
	gtk_icon_set_add_source(nufw_icon_active,src_nufw_icon_active);
	gtk_icon_set_ref(nufw_icon_active);

	gtk_icon_factory_add(nufw_icons,
			"nufw-inactive",
			nufw_icon);
	gtk_icon_factory_add(nufw_icons,
			"nufw-active",
			nufw_icon_active);
	gtk_icon_factory_add_default(nufw_icons);

	event_box = gtk_event_box_new ();

	{
		gchar sizename[30];
		int panel_size = panel_applet_get_size(nuapplet->applet);
		snprintf(sizename,29,"panel size %d",panel_size);
		nuapplet->panel_size = gtk_icon_size_register     (sizename     ,
				panel_size,
				panel_size);
	}
	image = gtk_image_new_from_stock( "nufw-inactive",
			nuapplet->panel_size 
			);

	nuapplet->image=image;

	gtk_container_add (GTK_CONTAINER (event_box), image);
	nuapplet->event_box=event_box;

	/* set tooltip */
	nuapplet->tooltip=gtk_tooltips_new ();
	gtk_tooltips_set_tip (nuapplet->tooltip,
			nuapplet->event_box,
			_("NuFW authentication inactive"),
			_("Click right to activate"));

	g_signal_connect(G_OBJECT(applet), "change_size",
			G_CALLBACK(applet_change_size),
			NULL);

	/* Setup the menus */
	panel_applet_setup_menu_from_file (PANEL_APPLET (applet),
			NULL,
			"NuFWContextMenu.xml",
			NULL,
			nu_menu_verbs,
			applet);

	gtk_container_add (GTK_CONTAINER (applet), event_box);

	if (panel_applet_gconf_get_bool(PANEL_APPLET(nuapplet->applet), "have_settings", NULL))
	{	
		char * tmp;

		nuapplet->interval = panel_applet_gconf_get_int(nuapplet->applet, "interval", NULL);
		nuapplet->port = panel_applet_gconf_get_int(nuapplet->applet, "port", NULL);
		nuapplet->log_at_start = panel_applet_gconf_get_bool(nuapplet->applet, "log_at_start", NULL);

		tmp = panel_applet_gconf_get_string(nuapplet->applet, "username", NULL);
		{
			nuapplet->username = g_strdup(tmp);
			g_free(tmp);
		}
		tmp = panel_applet_gconf_get_string(applet, "hostname", NULL);
		{
			nuapplet->host = g_strdup(tmp);
			g_free(tmp);
		}

	}
	gtk_widget_show_all (GTK_WIDGET (applet));
	if (nuapplet->log_at_start){
		launch_nu_client (NULL,nuapplet->applet,
				NULL) ;
	}
	return TRUE;
}

PANEL_APPLET_BONOBO_FACTORY ("OAFIID:NuFWClientApplet_Factory",
		PANEL_TYPE_APPLET,
		"NuApplet Factory",
		"0",
		(PanelAppletFactoryCallback)nu_applet_fill,
		NULL);
